import smtplib
from email.mime import text
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.header import Header
from typing import Union

# onebot11 协议
from nonebot.adapters.onebot.v11 import Bot as V11Bot

from nonebot import logger
from .config import plugin_config
from .dataClass import MailConfig

mail_config: MailConfig = MailConfig(
    user=plugin_config.disconnect_notice_smtp_user,
    password=plugin_config.disconnect_notice_smtp_password,
    server=plugin_config.disconnect_notice_smtp_server,
    port=plugin_config.disconnect_notice_smtp_port,
    notice_email=plugin_config.disconnect_notice_notice_email,
)


def send_notice(bot: Union[V11Bot]):
    """整合发送通知消息"""
    bot_id = get_bot_id(bot)
    send_mail(bot_id)


def send_mail(bot_id: str, test: bool = False):
    """发送邮件通知"""
    # 邮箱凭据
    global mail_config
    # 参数校验
    if not mail_config.check_params():
        error = "邮件通知缺少配置参数，无法进行通知"
        logger.error(error)
        return error
    # 邮件正文
    subject = f"你的QQbot掉线了"
    content = f"你的QQbot账号: {bot_id} 掉线了，可能是被风控了，赶快去看看吧"
    if test:
        subject = f"掉线通知测试"
        content = f"这是一封掉线通知测试邮件，你bot并没有掉线"

    # 构造邮件内容
    message = MIMEMultipart("alternative")
    message["Subject"] = Header(subject, 'utf-8')
    message["From"] = mail_config.user
    message["To"] = mail_config.notice_email
    message.attach(MIMEText(content))
    # 连接SMTP服务器并发送邮件
    if mail_config.port == 465:
        server = smtplib.SMTP_SSL(mail_config.server, mail_config.port)
    else:
        # 25或其他
        server = smtplib.SMTP(mail_config.server, mail_config.port)
    try:
        server.login(mail_config.user, mail_config.password)
        server.sendmail(mail_config.user, mail_config.notice_email, message.as_string())
    except Exception as e:
        logger.error(f"邮件发送失败，错误信息如下{e}")
        return e
    logger.info("通知邮件发送成功!")
    return


def get_bot_id(bot: Union[V11Bot]) -> str:
    """获取bot_id"""
    bot_id = bot.self_id
    return bot_id



from typing import Union

from nonebot.internal.matcher import Matcher
from nonebot.plugin import PluginMetadata

# onebot11 协议
from nonebot.adapters.onebot.v11 import Bot as V11Bot
from nonebot import on_command
from nonebot.permission import SUPERUSER

from .config import driver, plugin_config, global_config
from .utils import send_notice, mail_config, send_mail

__plugin_meta__ = PluginMetadata(
    name="QQbot断连通知",
    description="QQbot断连时的通知插件",
    usage="""
    超级用户指令:
        断连通知测试:主动触发掉线通知测试
    """,
    type="application",
    # 发布必填，当前有效类型有：`library`（为其他插件编写提供功能），`application`（向机器人用户提供功能）。
    homepage="https://github.com/Cypas/nonebot_plugin_disconnect_notice",
    # 发布必填。
    supported_adapters={"~onebot.v11"},
)

notice_test = on_command("断连通知测试", permission=SUPERUSER)


@notice_test.handle()
async def _(matcher: Matcher):
    """主动触发掉线通知测试"""
    msg = "已发送测试邮件，如未收到请检查邮件垃圾箱"
    res = send_mail("114514", test=True)
    if res is not None:
        msg = f"测试邮件发送失败，请重新检查配置项参数正确性，错误信息为: {res}"
    await matcher.finish(msg)


@driver.on_bot_disconnect
async def disconnect(bot: Union[V11Bot]):
    """bot断连触发器"""
    send_notice(bot)


@driver.on_bot_connect
async def connect(bot: V11Bot):
    """插件载入时检测配置完整性"""
    if not mail_config.check_params():
        # 缺少参数,私聊通知主人
        super_user: str = list(global_config.superusers)[0]
        if super_user:
            await bot.send_private_msg(user_id=int(super_user), message="【插件nonebot-plugin-disconnect-notice】\n"
                                                                        "缺少mail配置项，请按照https://github.com"
                                                                        "/Cypas/nonebot_plugin_disconnect_notice#%EF%B8%8F-%E9%85%8D%E7%BD%AE"
                                                                        "\n添加配置项后，再重新载入插件")



class MailConfig:
    def __init__(self,
                 user: str,
                 password: str,
                 server: str,
                 port: int,
                 notice_email: str):
        self.user = user
        self.password = password
        self.server = server
        self.port = port
        self.notice_email = notice_email

    def check_params(self) -> bool:
        """检查参数是否填写完整"""
        if self.user and self.password and self.server and self.port and self.notice_email:
            return True
        else:
            return False



from typing import List, Optional

from nonebot import get_driver
from pydantic import BaseModel


# 其他地方出现的类似 from .. import config，均是从 __init__.py 导入的 Config 实例
class Config(BaseModel):
    disconnect_notice_smtp_user = ""
    disconnect_notice_smtp_password = ""
    disconnect_notice_smtp_server = ""
    disconnect_notice_smtp_port = 465
    disconnect_notice_notice_email = ""


driver = get_driver()
# 本地测试时由于不启动 driver，需要将下面两行注释并取消再下一行的注释
global_config = driver.config
plugin_config = Config.parse_obj(global_config)

# plugin_config = Config()
